/************************************************************************
 * This macro assigns both dimensions of the 2006 canadian         ******
 * deprivation index to any administrative file with a postal      ******
 * code variable and, optionally, a census subdivision variable.   ******
 * Four versions, canadian, regional, metropolitan and by zone     ******
 * zone, are assigned. A simplified version of the PCCF+, a        ******
 * Statistics Canada product, is used to link the dissemination    ******
 * area with the postal code. More specifically, three             ******
 * conversion tables were used:                                    ******
 *  - WCF from the PCCF+                                           ******
 *  - PCCFDUPS from the PCCF+                                      ******
 *  - PCCFUNIQ from the PCCF+                                      ******
 *                                                                 ******
 * Macro parameters:                                               ******
 *  - IN: SAS input file on which the user wants to add the        ******
 *        material and social deprivation index (4 versions)       ******
 *  - PCODE: six-character postal code variable name               ******
 *  - MUNIC: census subdivison variable name. By default,          ******
 *           0 = the index is assigned with the postal code only   ******
 *  - OUT: SAS output file name,                                   ******
 *                                                                 ******
 * Authors: Philippe Gamache - philippe.gamache@inspq.qc.ca        ******
 * Last modified: November 27th                                    ******
 * Modifications: English version finalized                        ******
 ************************************************************************/

filename fcp "X:\Your Directory\fcp.txt";
filename pccfuniq "X:\Your Directory\fccpindiceuniq.txt";
filename pccfdups "X:\Your Directory\fccpindicedouble.txt";
filename munic "X:\Your Directory\munic.txt";
libname i2006 "X:\Your Directory";


data i2006.wcf;
infile fcp;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51 
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.pccfuniq;
infile pccfuniq;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.pccfdups;
infile pccfdups;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.munic;
  infile munic;
  input munic 1-7 mun96 8-14 mun01 15-21;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));

%end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;


/**** Do not change de following macro *****/
%macro assignationInd_2006_eng(in,pcode,munic,out);

%if %bquote(&munic)=%str() %then %let munic=0; ;

data wcf;
   set i2006.wcf;
   nuniq = _n_;
run;

data pccfuniq;
   set i2006.pccfuniq;
   nuniq = _n_;
run;

data pccfdups;
   set i2006.pccfdups;
   nuniq = _n_;
run;

  data fichin;
    %if &munic ne 0 %then %do; length munic 8.; %end;
    set &in;
    numero=_n_;
	nba=ranuni(766);

	pcode=compress(&pcode);
	%if &munic ne 0 %then 
     %do; 
       if substr(pcode,1,1) in ("A") then munic=1000000+mod(&munic,1000000);
       else if substr(pcode,1,1) in ("B") then munic=1200000+mod(&munic,1200000);
       else if substr(pcode,1,1) in ("C") then munic=1100000+mod(&munic,1100000);
       else if substr(pcode,1,1) in ("E") then munic=1300000+mod(&munic,1300000);
       else if substr(pcode,1,1) in ("G","H","J") then munic=2400000+mod(&munic,2400000);
       else if substr(pcode,1,1) in ("K","L","M","N","P") then munic=3500000+mod(&munic,3500000);
       else if substr(pcode,1,1) in ("R") then munic=4600000+mod(&munic,4600000);
       else if substr(pcode,1,1) in ("S") then munic=4700000+mod(&munic,4700000);
       else if substr(pcode,1,1) in ("T") then munic=4800000+mod(&munic,4800000);
       else if substr(pcode,1,1) in ("V") then munic=5900000+mod(&munic,5900000);
       else if substr(pcode,1,1) in ("X") then munic=6100000+mod(&munic,6100000);
       else if substr(pcode,1,1) in ("Y") then munic=6000000+mod(&munic,6000000);
     %end;
   run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
   data fichin;
   set fichin;
   where zone in (.,0);
   run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;

  proc sort data=i2006.munic;
    by munic;
   run;

  proc sort data=fichin;
    by munic;
   run;

  data fichin;
    merge i2006.munic(in=in2) fichin(in=in1);
	by munic;
	if in1*in2 then munic=mun01;
	else if in2 then delete;
   run;

/*********************************************************
  * Deprivation index assignment with the postal code  ***
  * and the census subdivision variables PCCF+ file    ***
  *******************************************************/


/*** With WCF *****/

   proc sql;
     create table test as
     select fichin.pcode, fichin.munic, numero, nba, wcf.propmunic, 
	        wcf.diffmatmunic, wcf.diffsocmunic, wcf.fortcp, wcf.nuniq,
            wcf.fortmunic, wcf.quintmat, wcf.quintsoc, wcf.centmat, wcf.centsoc, wcf.zim, wcf.zone,
			wcf.pr, wcf.quintmatRC, wcf.quintsocRC, wcf.quintmatRMR, wcf.quintsocRMR, wcf.quintmatZONE, wcf.quintsocZONE
     from fichin, wcf
     where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

   /*** Choice of one deprivation index if there's more than on postal code linked ***/

  data test;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appmunic1 nonapp1;
    merge fichin(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appmunic1;
	else if in1 then output nonapp1;
   run;

 /*** With PCCFUNIQ *****/

  proc sort data=nonapp1;
    by pcode munic;
   run;

  proc sort data=pccfuniq;
    by pcode munic;
   run;

   proc sql;
     create table appmunic2 as
     select nonapp1.pcode, nonapp1.munic, numero, nba, pccfuniq.propmunic, 
	        pccfuniq.diffmatmunic, pccfuniq.diffsocmunic, pccfuniq.fortcp,
            pccfuniq.fortmunic, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc,
			pccfuniq.zim, pccfuniq.zone,pccfuniq.nuniq,
			pccfuniq.pr, pccfuniq.quintmatRC, pccfuniq.quintsocRC, pccfuniq.quintmatRMR, pccfuniq.quintsocRMR, 
            pccfuniq.quintmatZONE, pccfuniq.quintsocZONE
     from nonapp1, pccfuniq
     where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
	 order by numero,nuniq; 
    quit;

  data nonapp2;
    merge nonapp1(in=in2) pccfuniq(in=in1);
	by pcode munic;
	if in2 and not(in1);
   run;

 /*** With PCCFDUPS *****/

   proc sql;
     create table test as
     select nonapp2.pcode, nonapp2.munic, numero, nba, pccfdups.propmunic, 
	        pccfdups.diffmatmunic, pccfdups.diffsocmunic, pccfdups.fortcp,pccfdups.nuniq,
            pccfdups.fortmunic, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.centmat, pccfdups.centsoc, pccfdups.zim, pccfdups.zone,
			pccfdups.pr, pccfdups.quintmatRC, pccfdups.quintsocRC, pccfdups.quintmatRMR, pccfdups.quintsocRMR,
            pccfdups.quintmatZONE, pccfdups.quintsocZONE
     from nonapp2, pccfdups
     where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

   /*** Choice of one deprivation index if there's more than on postal code linked ***/

  data test;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=nonapp2;
    by numero;
   run; 

  data appmunic3 fichin;
    merge nonapp2(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appmunic3;
	else if in1 then output fichin;
   run;

 %end;

/*********************************************************
  * Deprivation index assignation with the postal code ***
  * variable only                                      ***
  *******************************************************/
/*** With WCF *****/
  data wcf2;
    set wcf;
   run;

     proc sql;
     create table test as
     select fichin.pcode, fichin.numero, nba,
            wcf2.propcp, wcf2.diffmatcp, wcf2.diffsoccp, 
            wcf2.fortmunic, wcf2.fortcp,wcf2.nuniq,
			wcf2.quintmat, wcf2.quintsoc, wcf2.centmat, wcf2.centsoc,
			wcf2.zim, wcf2.zone, wcf2.pr, 
			wcf2.quintmatRC, wcf2.quintsocRC, wcf2.quintmatRMR, wcf2.quintsocRMR, wcf2.quintmatZONE, wcf2.quintsocZONE
     from fichin, wcf2
     where fichin.pcode=wcf2.pcode
	 order by numero,nuniq; 
   quit;

   proc sort data=test;
    by pcode numero;
   run;

   /*** Choice of one deprivation index if there's more than on postal code linked ***/

  data test;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appcp1 nonapp3;
    merge fichin(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appcp1;
	else if in1 then output nonapp3;
   run;

 /*** With PCCFUINIQ *****/
  proc sort data=nonapp3;
    by pcode;
   run;

  proc sort data=pccfuniq;
    by pcode;
   run;

     proc sql;
     create table appcp2 as
     select nonapp3.pcode, nonapp3.numero, nba,
            pccfuniq.propcp, pccfuniq.diffmatcp, pccfuniq.diffsoccp, 
            pccfuniq.fortmunic, pccfuniq.fortcp,pccfuniq.nuniq,
			pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc,
			pccfuniq.zim, pccfuniq.zone, pccfuniq.pr, 
			pccfuniq.quintmatRC, pccfuniq.quintsocRC, pccfuniq.quintmatRMR, pccfuniq.quintsocRMR,
			pccfuniq.quintmatZONE, pccfuniq.quintsocZONE
     from nonapp3, pccfuniq
     where nonapp3.pcode=pccfuniq.pcode
	 order by numero,nuniq; 
   quit;


  data nonapp4 ;
    merge nonapp3(in=in2) pccfuniq(in=in1);
	by pcode;
	if in2 and not(in1);
   run;

 /*** With PCCFDUPS *****/

  data pccfdup2;
    set pccfdups;
   run;

     proc sql;
     create table test as
     select nonapp4.pcode, nonapp4.numero, nba,
            pccfdup2.propcp, pccfdup2.diffmatcp, pccfdup2.diffsoccp, 
            pccfdup2.fortmunic, pccfdup2.fortcp,pccfdup2.nuniq,
			pccfdup2.quintmat, pccfdup2.quintsoc, pccfdup2.centmat, pccfdup2.centsoc,
			pccfdup2.zim, pccfdup2.zone, pccfdup2.pr, 
			pccfdup2.quintmatRC, pccfdup2.quintsocRC, pccfdup2.quintmatRMR, pccfdup2.quintsocRMR,
			pccfdup2.quintmatZONE, pccfdup2.quintsocZONE
     from nonapp4, pccfdup2
     where nonapp4.pcode=pccfdup2.pcode
	 order by numero,nuniq; 
   quit;

   proc sort data=test;
    by pcode numero;
   run;

   /*** Choice of one deprivation index if there's more than on postal code linked ***/

  data test;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=nonapp4;
    by numero;
   run; 

  data appcp3 nonapp5 ;
    merge nonapp4(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appcp3;
	else if in1 then output nonapp5;
   run;

  data tout;
    set %if &munic ne 0 %then %do; 
                                appmunic1(in=in1) appmunic2(in=in2)
                                appmunic3(in=in3)
                              %end;
	appcp1(in=in4) appcp2(in=in5) appcp3(in=in6)
    nonapp5(in=in7);
	if in7 then direct=0;
	if in5 then do; direct=1;diffmatcp=0; 
                    diffsoccp=0;
				end;
	%if &munic ne 0 %then %do; 
	                           if in1 then base=1;
                               else if in2 then do; direct=1; diffmatmunic=0; 
                                               diffsocmunic=0; base=2;
										   end;
							   else if in3 then base=3;
							   diffmat=diffmatmunic;
	                           diffsoc=diffsocmunic;
                          %end;
	if in4 then base=1;
	else if in5 then base=2;
	else if in6 then base=3;

	if diffmatcp ne . then diffmat=diffmatcp;
 	if diffsoccp ne . then diffsoc=diffsoccp;

	 if in7=1 then do; quintmat = 0; quintsoc = 0; quintmatRC=0; quintsocRC=0; 
                       quintmatRMR=0; quintsocRMR=0;quintmatZONE=0; quintsocZONE=0;  
     end;
	 if quintmat=0 then groupe=6;
	 else if quintmat=. then groupe=5;
	 else if direct=1 then groupe=1;
     else if diffmat=0 and diffsoc=0 then groupe=2;
	 else if fortcp=1 or fortmunic=1 then groupe=3;
     else groupe=4;
     css=zim;
	keep numero base groupe quintmat quintsoc centmat centsoc quintmatRC quintsocRC quintmatRMR quintsocRMR quintmatZONE 
         quintsocZONE zone pr;
   run;

  proc sort; by numero; run; 

  data fichin;
     set &in;
	 numero=_n_;
    run;

   data tout;
     retain quintmat quintsoc centmat centsoc quintmatRC quintsocRC quintmatRMR quintsocRMR quintmatZONE quintsocZONE zone pr base groupe numero;
     set tout;
   run;

  data &out;
    merge fichin tout;
	by numero;
	DATE = '20092711';
	group = groupe;quintmatCR = quintmatRC;quintsocCR = quintsocRC;quintmatCMA = quintmatRMR;quintsocCMA = quintsocRMR;miz=css;
	format group fgroupe. base fbase.;
	drop numero groupe quintmatRC quintsocRC quintmatRMR quintsocRMR miz;
   run;

proc datasets library=work;
 delete wcf2 pccfdup2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appcp1 
       appcp2 appcp3 nonapp1 nonapp2 nonapp3 nonapp4 nonapp5;
 quit;
%mend assignationInd_2006_eng;


proc format;
  value fgroupe 1="Unique assignment, postal code (and census subdivision) - DA - Index"
  		        2="Unique assignment, postal code (and census subdivision) - Index - Multiple DA"
				3="Non unique or random assignment with one DA > 90% population"
				4="Other non unique assignment"
				5="Paired to one DA without an index"
				6="No pairing possible";
  value fbase 1="WCF"
			  2="PCFF unique"
			  3="PCCF dups";
 run;

   /***************************************************************
 * Example: %assignation(birth06, pc, cen_div, birth06v2);  ******
 * - The input file is WORK.birth06                         ******
 * - The postal code variable is PC.                        ******
 * - The census subdivision variable is cen_div.            ******
 *   (N.B.: is it important to put 0 if you don't want to   ******
 *   assign the index with the census subdivision variable. ******
 * - The output file is WORK.birth06V2 (identical from the  ******
 *   input file, where the deprivation index (quintiles,    ******
 *   centiles),  were added                                 ******
 * - NOTE: The results will differ if the order of the      ******
 *   data in the input file is not the same.                ******
 *****************************************************************/ 

libname toto "X:\Your Directory";

%assignationInd_2006_eng(in=birth06,pcode=pc,munic=cen_div,out=birth06v2);

